import javax.sound.sampled.*;
import javax.swing.*;
import javax.swing.plaf.metal.MetalTheme;

import java.awt.*;
import java.awt.event.*;
import java.io.*;

import laf.AquaMetalTheme;
import laf.BigContrastMetalTheme;
import laf.ContrastMetalTheme;
import laf.DemoMetalTheme;
import laf.GreenMetalTheme;
import laf.KhakiMetalTheme;
import laf.PropertiesMetalTheme;

public class SoundRecorder {
	AudioInputStream audioInputStream;

	String errStr;

	public File old_file;

	String fileName = "untitled";

	double duration, seconds;

	public JFrame main_frame;

	public JButton record;

	public JButton stop;

	public JButton save;

	public JTextField file_name;

	public Container c;

	File file;

	public static void updateDesign(String design) {
		if (design.endsWith(".kthm")) {
			try {
				MetalTheme special_design = new PropertiesMetalTheme(
						new FileInputStream(design));
				System.setProperty("sun.awt.noerasebackground", "true");
				javax.swing.plaf.metal.MetalLookAndFeel
						.setCurrentTheme(special_design);
			} catch (Exception E) {
				design = "Mac Design";
			}
		} else if (design.equals("Mac Design")) {
		} else if (design.equals("Metal")) {
			System.setProperty("sun.awt.noerasebackground", "true");
			javax.swing.plaf.metal.MetalLookAndFeel
					.setCurrentTheme(new javax.swing.plaf.metal.DefaultMetalTheme());

		} else if (design.equals("Metal Green")) {
			System.setProperty("sun.awt.noerasebackground", "true");
			javax.swing.plaf.metal.MetalLookAndFeel
					.setCurrentTheme(new GreenMetalTheme());
		} else if (design.equals("Metal Aqua")) {
			System.setProperty("sun.awt.noerasebackground", "true");
			javax.swing.plaf.metal.MetalLookAndFeel
					.setCurrentTheme(new AquaMetalTheme());
		} else if (design.equals("Metal Khaki")) {
			System.setProperty("sun.awt.noerasebackground", "true");
			javax.swing.plaf.metal.MetalLookAndFeel
					.setCurrentTheme(new KhakiMetalTheme());
		} else if (design.equals("Metal Demo")) {
			System.setProperty("sun.awt.noerasebackground", "true");
			javax.swing.plaf.metal.MetalLookAndFeel
					.setCurrentTheme(new DemoMetalTheme());
		} else if (design.equals("Metal Contrast")) {
			System.setProperty("sun.awt.noerasebackground", "true");
			javax.swing.plaf.metal.MetalLookAndFeel
					.setCurrentTheme(new ContrastMetalTheme());
		} else if (design.equals("Metal Big Contrast")) {
			System.setProperty("sun.awt.noerasebackground", "true");
			javax.swing.plaf.metal.MetalLookAndFeel
					.setCurrentTheme(new BigContrastMetalTheme());
		} else {
			design = "Brush Metal";
			GUIManager.setBrushMetalLook(true);
		}
		if (!design.equals("Brush Metal") && !design.equals("Mac Design")) {
			try {
				UIManager
						.setLookAndFeel("javax.swing.plaf.metal.MetalLookAndFeel");
			} catch (Exception E) {

			}
		}
	}

	public void initObjects() {
		main_frame = new JFrame("Kema Audio Recorder");
		main_frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		file_name = new JTextField();
		c = main_frame.getContentPane();
		file_name = new JTextField(40);
		save = new JButton("Save");
		save.setToolTipText("Save file (.AIFF or in .WAV)");
		record = new JButton("Start record");
	}

	public void start() {
		updateDesign("Metal Aqua");
		initObjects();
		s();
	}

	public void s() {
		main_frame.setSize(350, 80);
		main_frame.setLocation(400, 300);
		main_frame.getContentPane().setLayout(new FlowLayout());
		main_frame.getContentPane().add(record);
		main_frame.getContentPane().add(save);
		main_frame.setVisible(true);
		final Capture capture = new Capture();
		record.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				if (record.getText().startsWith("Start")) {
					record.setText("Stop");
					capture.start();
				} else {
					record.setText("Start Record");
					capture.stop();
				}
			}
		});
		save.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				JFileChooser jfc = new JFileChooser();
				
				if (old_file != null) {
					jfc.setSelectedFile(old_file);
				}
				jfc.showSaveDialog(main_frame);
				File f = jfc.getSelectedFile();
				old_file = f;
				if (f == null) {
					System.err.println("for " + f + "");
					return;
				}
				if(f.toString().toLowerCase().endsWith(".wav") ||
						f.toString().toLowerCase().endsWith(".wave")) {
					saveToFile(f + "", AudioFileFormat.Type.WAVE);
				} else if(f.toString().toLowerCase().endsWith(".aifc") ||
						f.toString().toLowerCase().endsWith(".aifc")) {
					saveToFile(f + "", AudioFileFormat.Type.AIFC);
				} else if(f.toString().toLowerCase().endsWith(".aif") ||
						f.toString().toLowerCase().endsWith(".aiff")) {
					saveToFile(f + "", AudioFileFormat.Type.AIFF);
				}
			}
		});
	}

	public static void main(String[] args) {
		new SoundRecorder().start();
	}

	public void createAudioInputStream(File file, boolean updateComponents) {
		if (file != null && file.isFile()) {
			try {
				this.file = file;
				errStr = null;
				audioInputStream = AudioSystem.getAudioInputStream(file);
				fileName = file.getName();
				long milliseconds = (long) ((audioInputStream.getFrameLength() * 1000) / audioInputStream
						.getFormat().getFrameRate());
				duration = milliseconds / 1000.0;
				if (updateComponents) {
				}
			} catch (Exception ex) {
				System.err.println(ex.toString());
			}
		} else {
			System.err.println("Audio file required.");
		}
	}

	class Capture implements Runnable {

		TargetDataLine line;

		Thread thread;

		public void start() {
			errStr = null;
			thread = new Thread(this);
			thread.setName("Capture");
			thread.start();
			System.out.println("Started.");
		}

		public void stop() {
			thread = null;
		}

		private void shutDown(String message) {
			if ((errStr = message) != null && thread != null) {
				thread = null;
				System.err.println(errStr);
			}
		}

		public void run() {
			duration = 0;
			audioInputStream = null;
			AudioFormat.Encoding encoding = AudioFormat.Encoding.PCM_SIGNED;
			AudioFormat format = new AudioFormat(encoding, 44100f, 16, 2,
					(16 / 8) * 2, 44100f, true);
			DataLine.Info info = new DataLine.Info(TargetDataLine.class, format);

			if (!AudioSystem.isLineSupported(info)) {
				shutDown("Line matching " + info + " not supported.");
				return;
			}
			try {
				line = (TargetDataLine) AudioSystem.getLine(info);
				line.open(format, line.getBufferSize());
			} catch (LineUnavailableException ex) {
				shutDown("Unable to open the line: " + ex);
				return;
			} catch (SecurityException ex) {
				shutDown(ex.toString());
				return;
			} catch (Exception ex) {
				shutDown(ex.toString());
				return;
			}
			ByteArrayOutputStream out = new ByteArrayOutputStream();
			int frameSizeInBytes = format.getFrameSize();
			int bufferLengthInFrames = line.getBufferSize() / 8;
			int bufferLengthInBytes = bufferLengthInFrames * frameSizeInBytes;
			byte[] data = new byte[bufferLengthInBytes];
			int numBytesRead;
			line.start();
			while (thread != null) {
				if ((numBytesRead = line.read(data, 0, bufferLengthInBytes)) == -1) {
					break;
				}
				out.write(data, 0, numBytesRead);
			}
			line.stop();
			line.close();
			line = null;
			try {
				out.flush();
				out.close();
			} catch (IOException ex) {
				ex.printStackTrace();
			}
			byte audioBytes[] = out.toByteArray();
			ByteArrayInputStream bais = new ByteArrayInputStream(audioBytes);

			audioInputStream = new AudioInputStream(bais, format,
					audioBytes.length / frameSizeInBytes);

			System.out.println("Stopped.");
			long milliseconds = (long) ((audioInputStream
					.getFrameLength() * 1000) / format.getFrameRate());
			duration = milliseconds / 1000.0;

			try {
				audioInputStream.reset();
			} catch (Exception ex) {
				ex.printStackTrace();
				return;
			}
		}
	}

	public void saveToFile(String name, AudioFileFormat.Type fileType) {

		if (audioInputStream == null) {
			System.err
					.println("No audio to save null???????????????????????????????????????????????????????");
			return;
		} else if (file != null) {
			createAudioInputStream(file, false);
		}
		try {
			audioInputStream.reset();
		} catch (Exception e) {
			System.err.println("Unable to reset stream " + e);
			return;
		}

		File file = new File(fileName = name);
		try {
			if (AudioSystem.write(audioInputStream, fileType, file) == -1) {
				throw new IOException("Problems writing to file");
			}
		} catch (Exception ex) {
			System.err.println(ex.toString());
		}
	}
}
